/*********************************************************************
 *                
 * Filename:      obex_main.h
 * Version:       
 * Description:   
 * Status:        Experimental.
 * Author:        Dag Brattli <dagb@cs.uit.no>
 * Created at:    Mon Jul 20 22:28:23 1998
 * Modified at:   Tue Sep  7 22:39:39 1999
 * Modified by:   Dag Brattli <dagb@cs.uit.no>
 * 
 *     Copyright (c) 1998, 1999, Dag Brattli, All Rights Reserved.
 *      
 *     This library is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU Lesser General Public
 *     License as published by the Free Software Foundation; either
 *     version 2 of the License, or (at your option) any later version.
 *
 *     This library is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *     Lesser General Public License for more details.
 *
 *     You should have received a copy of the GNU Lesser General Public
 *     License along with this library; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *     MA  02111-1307  USA
 *     
 ********************************************************************/

#ifndef OBEX_MAIN_H
#define OBEX_MAIN_H

#include <time.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <sys/types.h>

#include "obex_object.h"
#include "obex_transport.h"

#include <gnetbuf.h>

/* use 0 for production, 1 for verification, >2 for debug */
#ifndef NET_DEBUG
#define NET_DEBUG 4
#endif
#ifdef NET_DEBUG
unsigned int obex_net_debug;
#define DEBUG(n, args...) if (obex_net_debug >= (n)) g_print(args)
#else
#define DEBUG(n, args...)
#endif /* NET_DEBUG */

#define OBEX_VERSION      0x11      /* Version 1.1 */
#define OBEX_DEFAULT_MTU  4096
#define OBEX_MINIMUM_MTU  255      

#define OBEX_TX 0
#define OBEX_RX 1

/* Standard headers */
#define HEADER_COUNT       0xc0 /* Number of objects (used by connect) */
#define HEADER_NAME        0x01 /* Name of the object */
#define HEADER_TYPE        0x42 /* Type of the object */
#define HEADER_TIME        0x44 /* Last modification of object */
#define HEADER_TIME2       0xC4 /* This value is also allowed for time */
#define HEADER_LENGTH      0xc3 /* Total lenght of object */
#define HEADER_DESCRIPTION 0x05 /* Description of object */
#define HEADER_TARGET      0x46 /* Identifies the target for the object */
#define HEADER_BODY        0x48 /* Data part of the object */
#define HEADER_BODY_END    0x49 /* Last data part of the object */
#define HEADER_WHO         0x4a /* Identifies the sender of the object */
#define HEADER_CONNECTION  0xcb /* Connection identifier */

#define OBEX_FINAL         0x80
#define OBEX_CMD           1
#define OBEX_RSP           0
#define OBEX_SYNC          0
#define OBEX_ASYNC         1

/* Commands */
#define CMD_CONNECT                0x00
#define CMD_DISCONNECT             0x01
#define CMD_PUT                    0x02
#define CMD_GET                    0x03
#define CMD_COMMAND                0x04
#define CMD_SETPATH                0x05
#define CMD_ABORT                  0x7f

/* Responses */
#define	OBEX_CONTINUE              0x10
#define OBEX_SWITCH_PRO            0x11
#define OBEX_SUCCESS               0x20
#define OBEX_CREATED               0x21
#define OBEX_ACCEPTED              0x22
#define OBEX_NO_CONTENT            0x24
#define OBEX_BAD_REQUEST           0x40
#define OBEX_UNAUTHORIZED          0x41
#define OBEX_PAYMENT_REQUIRED      0x42
#define OBEX_FORBIDDEN             0x43
#define OBEX_NOT_FOUND             0x44
#define OBEX_METHOD_NOT_ALLOWED    0x45
#define OBEX_CONFLICT              0x49
#define OBEX_INTERNAL_SERVER_ERROR 0x50
#define OBEX_NOT_IMPLEMENTED       0x51
#define OBEX_DATABASE_FULL         0x60
#define OBEX_DATABASE_LOCKED       0x61

typedef struct {
	gint   total;      /* Total bytes to transfer */
	gint   current;    /* Currently transfered bytes */
	gfloat progress;   /* A number between 0 and 1 */
	gint   bps;        /* Estimated throughput */
	gint   seconds;    /* Seconds currently used by transfer */
} obex_status_t;


/* Forward decl */
struct obex;
typedef struct obex obex_t;

typedef struct {
	obex_object_t* (*get_indication)(obex_object_t *obj);
	void (*object_indication)(obex_object_t *obj, int response);
	void (*status_indication)(obex_status_t *status, int response);
	void (*connect_indication)(obex_t *self, GNetBuf *target);
	void (*connect_confirm)(obex_t *self);
	void (*disconnect_indication)(obex_t *self);
	int  (*setpath_indication)(char* path, unsigned char flags, 
				   unsigned char constants);
} obex_notify_t;

struct obex {
	volatile gint state;
	volatile gint put_state;  /* Current state during a put command */
	gint direction;           /* Direction of transfer */

	guint16 mtu_tx;		  /* Maximum OBEX Tx packet size */
        guint16 mtu_rx;           /* Maximum OBEX Rx packet size */

	gint fd;                  /* Socket descriptor */
	gint async;               /* True if we are working async */
	
	GNetBuf *tx_msg;          /* Reusable transmit message */
	GNetBuf *rx_msg;          /* Reusable receive message */

	struct timeval  trans_start;

	gint            count;    /* Numbers of objects in this transaction */
	GNetBuf         *target;
	GNetBuf         *who;
	obex_object_t   *object;  /* Current object being transfered */      
        obex_status_t   status;   /* Status of transfer */

	gint            response_code;  /* Last response opcode */
	volatile int    response_next;  /* Next indication a response? */

	obex_notify_t   notify;   /* Callbacks to application */

	gboolean connected;       /* OBEX connection state */
	obex_transport_t trans;   /* Transport being used */
};

void input_handler(int signal);
gint obex_data_indication(obex_t *self);


void obex_object_indication(obex_t *self, gint response);
void obex_status_indication(obex_t *self, gint response);
void obex_clear_status(obex_t *self);

gint obex_data_request(obex_t *self, GNetBuf *msg, gint opcode, 
		       gint response);

int obex_setpath_indication(obex_t *self, GNetBuf *msg, int final);

void obex_response_request(obex_t *self, guint8 opcode);
gint obex_response_indication(obex_t *self, GNetBuf *msg);

GString *obex_get_last_response_message(obex_t *self);

static inline void obex_set_response_code(obex_t *self, gint code)
{
	self->response_code = code;
}

static inline gint obex_get_response_code(obex_t *self)
{
	return self->response_code;
}

gint obex_transport_connect_request(obex_t *self);
void obex_transport_disconnect_request(obex_t *self);
gint obex_transport_listen(obex_t *self, char *service);

extern obex_t *async_self;

#endif

